@@ -267,7 +267,7 @@ use Glib::Object::Subclass Gscan2pdf::Dialog::, signals => {
     ),
   ];
 
-our $VERSION = '2.1.6';
+our $VERSION = '2.2.2';
 
 my ( $d_sane, $logger );
 my $tolerance             = 1;
@@ -649,7 +649,7 @@ sub _save_profile_callback {
                         'gtk-cancel' => 'cancel'
                     );
                     $label = Gtk3::Label->new($warning);
-                    $dialog2->vbox->add($label);
+                    $dialog2->get_content_area->add($label);
                     $label->show;
                     if ( $dialog2->run eq 'ok' ) {
                         $parent->save_current_profile( $entry->get_text );
@@ -807,7 +807,7 @@ sub _flatbed_or_duplex_callback {
     my ($self) = @_;
     my $options = $self->get('available-scan-options');
     if ( defined $options ) {
-        if ( $self->_flatbed_selected($options) or $options->can_duplex ) {
+        if ( $options->flatbed_selected or $options->can_duplex ) {
             $self->{vboxx}->hide;
         }
         else {
@@ -825,13 +825,14 @@ sub _changed_scan_option_callback {
         and $name eq $options->{source}{name} )
     {
         if ( $self->get('allow-batch-flatbed')
-            or not $self->_flatbed_selected($options) )
+            or not $options->flatbed_selected )
         {
             $self->{framen}->set_sensitive(TRUE);
         }
         else {
             $bscannum->set_active(TRUE);
             $self->set( 'num-pages', 1 );
+            $self->set( 'sided',     'single' );
             $self->{framen}->set_sensitive(FALSE);
         }
 
@@ -853,7 +854,7 @@ sub _set_allow_batch_flatbed {
     }
     else {
         my $options = $self->get('available-scan-options');
-        if ( $self->_flatbed_selected($options) ) {
+        if ( defined $options and $options->flatbed_selected ) {
             $self->{framen}->set_sensitive(FALSE);
 
             # emits changed-num-pages signal, allowing us to test
@@ -864,24 +865,10 @@ sub _set_allow_batch_flatbed {
     return;
 }
 
-sub _flatbed_selected {
-    my ( $self, $options ) = @_;
-    return (
-              defined $options
-          and defined $options->{source}
-          and ( defined $options->{source}{val}
-            and $options->{source}{val} =~ /flatbed/xsmi )
-          or (  $#{ $options->{source}{constraint} } == 0
-            and $options->{source}{constraint}[0] =~ /flatbed/xsmi )
-    );
-}
-
 sub _set_available_scan_options {
     my ( $self, $name, $newval ) = @_;
     $self->{$name} = $newval;
-    if ( not $self->get('allow-batch-flatbed')
-        and $self->_flatbed_selected($newval) )
-    {
+    if ( not $self->get('allow-batch-flatbed') and $newval->flatbed_selected ) {
         if ( $self->get('num-pages') != 1 ) { $self->set( 'num-pages', 1 ) }
         $self->{framen}->set_sensitive(FALSE);
     }
@@ -908,7 +895,7 @@ sub _set_num_pages {
         or $self->get('allow-batch-flatbed')
         or (    defined $options
             and defined $options->{source}{val}
-            and not $self->_flatbed_selected($options) )
+            and not $options->flatbed_selected )
       )
     {
         $self->{$name} = $newval;
@@ -1244,6 +1231,22 @@ sub update_options {
     # so don't set this until we have finished
     $self->set( 'available-scan-options', $new_options );
 
+    # Remove buttons from $current_scan_options to prevent buttons which cause
+    # reloads from setting off infinite loops
+    my @buttons = ();
+    my $iter    = $current_scan_options->each_backend_option;
+    while ( my $i = $iter->() ) {
+        my ( $name, $val ) =
+          $current_scan_options->get_backend_option_by_index($i);
+        my $opt = $options->by_name($name);
+        if ( defined( $opt->{type} ) and $opt->{type} == SANE_TYPE_BUTTON ) {
+            push @buttons, $name;
+        }
+    }
+    for my $button (@buttons) {
+        $current_scan_options->remove_backend_option_by_name($button);
+    }
+
     # Reapply current options to ensure the same values are still set.
     $self->set_current_scan_options($current_scan_options);
 
@@ -1800,11 +1803,10 @@ sub set_options {
         title           => $d_sane->get( $opt->{title} ),
         destroy         => TRUE,
     );
-    my $vbox   = $window->vbox;
     my $canvas = Goo::Canvas->new;
     $canvas->set_size_request( $CANVAS_SIZE, $CANVAS_SIZE );
     $canvas->{border} = $CANVAS_BORDER;
-    $vbox->add($canvas);
+    $window->get_content_area->add($canvas);
     my $root = $canvas->get_root_item;
 
     $canvas->signal_connect(
@@ -1862,13 +1864,9 @@ sub set_options {
           sort { $a <=> $b } @{ $opt->{constraint} };
     }
 
-    # HBox for buttons
-    my $hbox = Gtk3::HBox->new;
-    $vbox->pack_start( $hbox, FALSE, TRUE, 0 );
-
     # Apply button
     my $abutton = Gtk3::Button->new_from_stock('gtk-apply');
-    $hbox->pack_start( $abutton, TRUE, TRUE, 0 );
+    $window->add_button($abutton);
     $abutton->signal_connect(
         clicked => sub {
             $self->set_option( $opt, $canvas->{val} );
@@ -1882,7 +1880,7 @@ sub set_options {
 
     # Cancel button
     my $cbutton = Gtk3::Button->new_from_stock('gtk-cancel');
-    $hbox->pack_end( $cbutton, FALSE, FALSE, 0 );
+    $window->add_button($cbutton);
     $cbutton->signal_connect( clicked => sub { $window->destroy } );
 
 # Have to show the window before updating it otherwise is doesn't know how big it is
