@@ -16,7 +16,7 @@ Readonly my $EMPTY_ARRAY => -1;
 # Glib::ParamSpec->object in Gscan2pdf::Dialog::Scan
 use Glib::Object::Subclass Glib::Object::;
 
-our $VERSION = '2.1.6';
+our $VERSION = '2.2.2';
 
 my $units = qr{(pel|bit|mm|dpi|%|us)}xsm;
 my $EMPTY = q{};
@@ -223,6 +223,16 @@ sub can_duplex {
     return FALSE;
 }
 
+sub flatbed_selected {
+    my ($self) = @_;
+    return (
+        defined $self->{source} and ( defined $self->{source}{val}
+            and $self->{source}{val} =~ /flatbed/xsmi )
+          or (  $#{ $self->{source}{constraint} } == 0
+            and $self->{source}{constraint}[0] =~ /flatbed/xsmi )
+    );
+}
+
 # returns TRUE/FALSE if the value is within the tolerance of the given option or
 # not, and undef for options with no value or for an invalid value
 
